'Projekt-Einstellungen:
'Option Strict On
'Option Explicit On
'Imports System.Windows.Forms
'Imports System.Drawing
'Imports Microsoft.VisualBasic.ControlChars

Imports AdapterInfo = System.Collections.Generic.KeyValuePair(Of Object, System.Data.DataTable)

Public Class DatasetAdapter

    'Der DatasetAdapter setzt Beziehungen mit Lschweitergabe vorraus, sowohl in Datenbank als 
    'auch im Dataset.
    'Primrschlsselwerte werden clientseitig generiert, **nicht in der Datenbank**.
    'Autowerte drfen auch nicht im Dataset-Designer konfiguriert werden; auch das bernimmt der 
    'DatasetAdapter (late Initializing)
    'Andernfalls msste der Client fr jeden neuen Datensatz denselben zunchst in die Datenbank 
    'speichern, und sich dann den dort generierten Schlssel zurckgeben lassen, und im Datensatz
    'eintragen, denn ohne Primrschlssel ist das Dataset nicht arbeitsfhig.
    'Fr Multi-Client-Datenbanken ergibt sich daraus, da als Primrschlssel-Typ nur GUID in
    ' Frage kommt, ansonsten mehrere Clients evtl. denselben Primrschlssel-Wert generieren
    ' knnten (Aber Multi-Client-Datenbanken haben noch viel mehr Probleme zu lsen).


    'Die Verwendung von CallByName bei Load() und Save() ist natrlich recht unsauber. Trotzdem
    ' ist es sicherer, diese Datenzugriffs-Logik immer wieder zu verwenden, statt fr jedes
    ' Dataset eine neue zu Datenzugriffs-Logik implementieren.
    Private _AdapterInfos As New List(Of AdapterInfo)
    Private _Dataset As DataSet

    Public Sub Add(ByVal Adapter As Object, ByVal Table As DataTable)
        If _AdapterInfos.Count = 0 Then
            If _Dataset Is Nothing Then
                _Dataset = Table.DataSet
                InitAutoValues(_Dataset)
            End If
        ElseIf _Dataset IsNot Table.DataSet Then
            Throw New ArgumentException(String.Concat( _
               "Die DataTable ", Table.TableName, " entstammt einem anderen Dataset"))
        End If
        'bergeordnete Tabellen vorne einordnen
        Dim I As Integer
        For I = 0 To _AdapterInfos.Count - 1
            If IsParentOf(Table, _AdapterInfos(I).Value) Then Exit For
        Next
        _AdapterInfos.Insert(I, New AdapterInfo(Adapter, Table))
    End Sub

   Private Shared Function IsParentOf( _
           ByVal tbParent As DataTable, ByVal tbChild As DataTable) As Boolean
      For Each Rl As DataRelation In tbParent.ChildRelations
         If Rl.ChildTable Is tbChild Then Return True
      Next
      For Each Rl As DataRelation In tbParent.ChildRelations
         If Rl.ChildTable Is Rl.ParentTable Then ' Endlos-Rekursion vermeiden
         ElseIf IsParentOf(Rl.ChildTable, tbChild) Then
            Return True
         End If
      Next
      Return False
   End Function

   ''' <summary>richtet Generierung von Primrschlsselwerten beim Client ein</summary>
   ''' <remarks>
   ''' untersttzt Datentypen, die auf AutoIncrement gesetzt werden knnen, sowie System.Guid
   ''' </remarks>
   Private Sub InitAutoValues(ByVal DataSet As DataSet)
      For Each Tb As DataTable In DataSet.Tables
         Dim PrimKey As DataColumn() = Tb.PrimaryKey
         If PrimKey.Length <> 1 Then Continue For
         Dim Clmn As DataColumn = PrimKey(0)
         Dim Tp As Type = Clmn.DataType
         If Tp.Equals(GetType(Int16)) OrElse _
               Tp.Equals(GetType(Int32)) OrElse _
               Tp.Equals(GetType(Int64)) Then
            Clmn.AutoIncrement = True
         ElseIf Tp.Equals(GetType(Guid)) Then
            AddHandler Clmn.Table.TableNewRow, AddressOf Table_NewRow
         End If
      Next
   End Sub

   Private Sub Table_NewRow(ByVal sender As Object, ByVal e As DataTableNewRowEventArgs)
      e.Row(e.Row.Table.PrimaryKey(0)) = Guid.NewGuid
   End Sub

   Public Sub Load()
      'Dim SW As Stopwatch = Stopwatch.StartNew
      For I As Integer = _AdapterInfos.Count - 1 To 0 Step -1
         With _AdapterInfos(I).Value
            .BeginLoadData()
            .Clear()
         End With
      Next
      For Each Inf As AdapterInfo In _AdapterInfos
         CallByName(Inf.Key, "Fill", CallType.Method, Inf.Value)
         Inf.Value.EndLoadData()
      Next
      'Dbg(SW.ElapsedMilliseconds)
   End Sub


   Public Sub Save()
      'Lschweitergabe zur Erhaltung der Datenkonsistenz: Lschen einer bergeordneten
      ' Datarow lscht alle ChildRows mit
      'Problem beim Update: Update ich den DELETE einer bergeordneten Datarow, so wendet
      ' auch die Datenbank Lschweitergabe auf ihre Daten an. Update ich danach die DELETEs
      ' der ChildRows, so wirft die DB eine Exception, denn bei ihr sind diese Daten schon weg.
      '(Hier vorgetragene) Lsung: Zunchst nur die DELETEs aller bergeordneter Rows des
      ' ganzen Datasets updaten (Top -> Down). Dabei temporr fr alle Datarelations des
      ' Datasets AcceptRejectRule.Cascade einstellen. Wenn nun die bergeordneten geupdatet
      ' sind, und (intern) ihr AcceptChanges() aufrufen, akzeptieren die ChildRows auch.
      'Eine akzeptierte (Child-)Datarow gibt mit DataRowState.Unchanged an, da sie in 
      'bereinstimmung mit der Datenbank ist, und vom Update auszunehmen.
      SetAcceptReject(AcceptRejectRule.Cascade)
      For Each Inf As AdapterInfo In _AdapterInfos
         If Inf.Value.ChildRelations.Count > 0 Then
            Dim rwDeletes As Object = Inf.Value.Select("", "", DataViewRowState.Deleted)
            CallByName(Inf.Key, "Update", CallType.Method, rwDeletes)
         End If
      Next
      SetAcceptReject(AcceptRejectRule.None)
      'INSERTs, und UPDATEs normal updaten
      For Each Inf As AdapterInfo In _AdapterInfos
         CallByName(Inf.Key, "Update", CallType.Method, Inf.Value)
      Next
   End Sub

   Private Sub SetAcceptReject(ByVal Rule As AcceptRejectRule)
      For Each Inf As AdapterInfo In _AdapterInfos
         For Each Rel As DataRelation In Inf.Value.ChildRelations
            Rel.ChildKeyConstraint.AcceptRejectRule = Rule
         Next
      Next
   End Sub

End Class
