Imports System.ComponentModel
Imports DBLoadSave.BestellungDataSet

Public Class frmDBLoadSave

    Private _Observating As Boolean

#Region "DatasetAdapter"
    'Diese Region enthlt alles, was man fr den Datenzugriff auch komplizierterer Datasets
    ' braucht. Sogar mehr: Alles, was mit _Observating zu tun hat gehrt eigentlich schon in die
    ' Region "Bestell-Funktionalitt" 

    Private _DatasetAdapter As New DatasetAdapter

    Private Sub frmDBLoadSave_Load(ByVal sender As Object, ByVal e As EventArgs) _
            Handles MyBase.Load
        With Me.BestellungDataSet
            _DatasetAdapter.Add(KundeTableAdapter, .Kunde)
            _DatasetAdapter.Add(BestellpostenTableAdapter, .Bestellposten)
            _DatasetAdapter.Add(BestellungTableAdapter, .Bestellung)

            'unvernderliche Tabellen kann ich auch vom DatasetAdapter ausnehmen, und
            ' einmalig herkmmlich befllen. Hierbei gilt: Parent-Tables first!!
            KategorieTableAdapter.Fill(.Kategorie)
            ArtikelTableAdapter.Fill(.Artikel)
        End With
        _DatasetAdapter.Load()
        'Bug-Workaround mittm Databinding der TabPage "tpSelectArtikel"
        Me.TabControl1.SelectedIndex = 2
        Me.TabControl1.SelectedIndex = 0
    End Sub

    Private Sub Button_Click(ByVal sender As Object, ByVal e As EventArgs) _
            Handles btReLoad.Click, btSave.Click, btManual.Click
        Dim Observating As Boolean = _Observating
        EnableArticleObserver(False)
        Select Case True
            Case sender Is btReLoad
                _DatasetAdapter.Load()
            Case sender Is btSave
                _DatasetAdapter.Save()
            Case sender Is btManual
                MsgBox(My.Settings.Manual)
        End Select
        EnableArticleObserver(Observating)
    End Sub

#End Region 'DatasetAdapter

#Region "Verschiedenes"

    Private Sub BestellpostenSource_ListChanged( _
            ByVal sender As Object, ByVal e As ListChangedEventArgs) _
            Handles BestellpostenSource.ListChanged
        'Bei nderung eines Bestellpostens BestellSumme neu rechnen und anzeigen
        Static ValidChangeTypes As IList(Of ListChangedType) = New ListChangedType() { _
            ListChangedType.Reset, _
            ListChangedType.ItemChanged, _
            ListChangedType.ItemDeleted}
        If ValidChangeTypes.Contains(e.ListChangedType) Then
            Dim BestellSum As Decimal = 0D
            For Each DRV As DataRowView In BestellpostenSource
                Dim Posten As BestellpostenRow = DirectCast(DRV.Row, BestellpostenRow)
                BestellSum += Posten.Preis
            Next
            AssignSave(lbBestellSum.Text, BestellSum.ToString("c"))
        End If
    End Sub

    Private Sub BestellungGrid_CellEnter( _
            ByVal sender As Object, ByVal e As DataGridViewCellEventArgs) _
            Handles BestellungGrid.CellEnter
        If e.RowIndex = BestellungGrid.NewRowIndex Then
            'Betreten der Hinzufge-Zeile bernimmt sofort den neuen Datensatz mit aktuellem
            ' Datum, ohne auf eine User-Eingabe zu warten
            Dim Rw As BestellungRow = GetRow(Of BestellungRow)(BestellungSource.Current)
            Rw.Bestelldatum = Date.Today
            Rw.Table.Rows.Add(Rw)
        End If
    End Sub

#End Region 'Verschiedenes

#Region "Bestell-Funktionalitt"

    'Ab hier gehts um das recht trickreiche GUI: Gibt man im Artikel-Katalog bei "AnzahlInput"
    ' eine Zahl ein, so wird automatisch der aktuellen Bestellung ein Bestellposten hinzufgt. 
    'Das funktioniert so: "Artikel" ist eine "Bestellposten" bergeordnete DataTable
    'Die enthaltene Datenspalte "Artikel.AnzahlInput" ist im Dataset-Designer hinzugefgt,
    ' und wird nicht mit abgespeichert.
    'Die dortigen Werte werden laufend synchronisiert mit den Werten der "ordentlichen"
    ' Datenspalte "Bestellposten.Anzahl" (die abgespeichert wird).
    'So erhalte ich o.g. Verhalten: User kann in allen Artikeln blttern, und dabei seine
    ' aktuelle Bestellung editieren, indem er nur die gewnschte Anzahl dazu-schreibt, 
    ' und ohne den View verlassen zu mssen

    Private Sub TabControl1_Selected(ByVal sender As Object, ByVal e As TabControlEventArgs) _
            Handles TabControl1.Selected
        'grundlegende Synchronisation beim Wechsel der TabPages
        EnableArticleObserver(False)
        Dim Posten As BestellpostenRow
        Select Case True
            Case e.TabPage Is tpKunde
                Return
            Case e.TabPage Is tpSelectArtikel
                'schreibe die aktuellen "Bestellposten.Anzahl" nach "Artikel.AnzahlInput" 
                For Each DRV As DataRowView In BestellpostenSource
                    Posten = DirectCast(DRV.Row, BestellpostenRow)
                    Posten.ArtikelRow.AnzahlInput = Posten.Anzahl
                Next
            Case e.TabPage Is tpBestellung
                'lsche die temporren Eintrge der bergeordneten "Artikel.AnzahlInput"
                For Each DRV As DataRowView In BestellpostenSource
                    Posten = DirectCast(DRV.Row, BestellpostenRow)
                    Posten.ArtikelRow.SetAnzahlInputNull()
                Next
        End Select
        EnableArticleObserver(True)
    End Sub

    Private Sub ArticleObserver_RowChanged( _
            ByVal sender As Object, ByVal e As BestellungDataSet.ArtikelRowChangeEvent)
        If e.Action = DataRowAction.Commit Then Return
        'User hat einen Eintrag in "Artikel.AnzahlInput" gemacht
        EnableArticleObserver(False)
        Dim Artikel As ArtikelRow = e.Row
        Dim Posten As BestellpostenRow
        Dim HasAnzahl As Boolean = Not Artikel.IsAnzahlInputNull AndAlso Artikel.AnzahlInput > 0
        Dim I As Integer = BindingSourceFind(BestellpostenSource, "ArtikelID", Artikel.ArtikelID)
        If I < 0 Then
            If HasAnzahl Then
                Posten = GetRow(Of BestellpostenRow)(BestellpostenSource.AddNew())
                Posten.ArtikelRow = Artikel
                Posten.Anzahl = Artikel.AnzahlInput
                BestellpostenSource.EndEdit()
            End If
        Else
            Posten = GetRow(Of BestellpostenRow)(BestellpostenSource(I))
            If HasAnzahl Then
                AssignSave(Posten.Anzahl, Artikel.AnzahlInput)
            Else
                Posten.Delete()
                BestellpostenSource.EndEdit()
            End If
        End If
        EnableArticleObserver(True)
    End Sub

    Private Sub EnableArticleObserver(ByVal Enable As Boolean)
        If _Observating = Enable Then Return
        _Observating = Enable
        If Enable Then
            AddHandler Me.BestellungDataSet.Artikel.ArtikelRowChanged, _
                AddressOf ArticleObserver_RowChanged
        Else
            RemoveHandler Me.BestellungDataSet.Artikel.ArtikelRowChanged, _
                AddressOf ArticleObserver_RowChanged
        End If
    End Sub

#End Region 'Bestell-Funktionalitt

End Class
